@@ -52,7 +52,6 @@ struct _LiPkgBuilderPrivate
 };
 
 G_DEFINE_TYPE_WITH_PRIVATE (LiPkgBuilder, li_pkg_builder, G_TYPE_OBJECT)
-
 #define GET_PRIVATE(o) (li_pkg_builder_get_instance_private (o))
 
 /**
@@ -401,8 +400,8 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 		g_set_error (error,
 			LI_BUILDER_ERROR,
 			LI_BUILDER_ERROR_SIGN,
-			_("Signing of package failed: %s"),
-			gpgme_strsource (err));
+			_("Signing of package failed (init): %s"),
+			gpgme_strerror (err));
 		gpgme_release (ctx);
 		return NULL;
 	}
@@ -418,8 +417,8 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 			g_set_error (error,
 				LI_BUILDER_ERROR,
 				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
+				_("Signing of package failed (get-key): %s"),
+				gpgme_strerror (err));
 			gpgme_release (ctx);
 			return NULL;
 		}
@@ -429,8 +428,8 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 			g_set_error (error,
 				LI_BUILDER_ERROR,
 				LI_BUILDER_ERROR_SIGN,
-				_("Signing of package failed: %s"),
-				gpgme_strsource (err));
+				_("Signing of package failed (signers-add): %s"),
+				gpgme_strerror (err));
 			gpgme_release (ctx);
 			return NULL;
 		}
@@ -444,7 +443,7 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 			LI_BUILDER_ERROR,
 			LI_BUILDER_ERROR_SIGN,
 			_("Signing of package failed: %s"),
-			gpgme_strsource (err));
+			gpgme_strerror (err));
 		gpgme_data_release (din);
 		gpgme_release (ctx);
 		return NULL;
@@ -456,7 +455,7 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 			LI_BUILDER_ERROR,
 			LI_BUILDER_ERROR_SIGN,
 			_("Signing of package failed: %s"),
-			gpgme_strsource (err));
+			gpgme_strerror (err));
 		gpgme_data_release (din);
 		gpgme_release (ctx);
 		return NULL;
@@ -474,8 +473,8 @@ li_pkg_builder_sign_data (LiPkgBuilder *builder, const gchar *data, gpgme_sig_mo
 		g_set_error (error,
 			LI_BUILDER_ERROR,
 			LI_BUILDER_ERROR_SIGN,
-			_("Signing of package failed: %s"),
-			gpgme_strsource (err));
+			_("Signing of package failed (sign): %s"),
+			gpgme_strerror (err));
 		return NULL;
 	}
 
@@ -710,7 +709,7 @@ li_pkg_builder_build_package_with_details (LiPkgBuilder *builder, LiPkgInfo *ctl
 		tmp = g_build_filename (payload_root, "include", NULL);
 		if (!g_file_test (tmp, G_FILE_TEST_IS_DIR)) {
 			g_free (tmp);
-			tmp = g_build_filename (payload_root, "opt", "bundle", "include", NULL);
+			tmp = g_build_filename (payload_root, "app", "include", NULL);
 			if (!g_file_test (tmp, G_FILE_TEST_IS_DIR)) {
 				/* no headers? No automatically built SDK package! */
 				return TRUE;
