@@ -1,7 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
  * Copyright (C) 2014-2015 Matthias Klumpp <matthias@tenstral.net>
- * Copyright (C)      2012 Alexander Larsson <alexl@redhat.com>
+ * Copyright (C) 2012-2015 Alexander Larsson <alexl@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -22,155 +22,45 @@
 #define _GNU_SOURCE /* Required for CLONE_NEWNS */
 #include <config.h>
 #include <limba.h>
-#include <li-config-data.h>
-#include "li-utils-private.h"
+#include <li-utils-private.h>
+#include <li-run.h>
 
 #include <sys/mount.h>
-#include <sched.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
 #include <errno.h>
-#include <linux/version.h>
+#include <sys/stat.h>
 #include <sys/utsname.h>
-#include <sys/capability.h>
-#include <sys/prctl.h>
 
 #include <gio/gio.h>
 
-#define REQUIRED_CAPS (CAP_TO_MASK(CAP_SYS_ADMIN))
-
 /**
- * Ensure we have just the capabilities we need
- */
-static gboolean
-acquire_caps (void)
-{
-	struct __user_cap_header_struct hdr;
-	struct __user_cap_data_struct data;
-
-	if (getuid () != geteuid ()) {
-		/* Tell kernel not clear capabilities when dropping root */
-		if (prctl (PR_SET_KEEPCAPS, 1, 0, 0, 0) < 0) {
-			g_printerr ("prctl(PR_SET_KEEPCAPS) failed\n");
-			return FALSE;
-		}
-
-		/* Drop root uid, but retain the required permitted caps */
-		if (setuid (getuid ()) < 0) {
-			g_printerr ("unable to drop privileges\n");
-			return FALSE;
-		}
-	}
-
-	memset (&hdr, 0, sizeof(hdr));
-	hdr.version = _LINUX_CAPABILITY_VERSION;
-
-	/* Drop all non-require capabilities */
-	data.effective = REQUIRED_CAPS;
-	data.permitted = REQUIRED_CAPS;
-	data.inheritable = 0;
-	if (capset (&hdr, &data) < 0) {
-		g_printerr ("capset failed\n");
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-/**
- * drop_caps:
- *
- * Drop all remaining capabilities we have.
- */
-static gboolean
-drop_caps (void)
-{
-	struct __user_cap_header_struct hdr;
-	struct __user_cap_data_struct data;
-
-	memset (&hdr, 0, sizeof(hdr));
-	hdr.version = _LINUX_CAPABILITY_VERSION;
-	data.effective = 0;
-	data.permitted = 0;
-	data.inheritable = 0;
-
-	if (capset (&hdr, &data) < 0) {
-		g_printerr ("capset failed\n");
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-/**
- * create_mount_namespace:
+ * mount_app_bundle:
+ * @pkgid: A software identifier ("name/version")
  */
 static int
-create_mount_namespace (void)
-{
-	int mount_count;
-	int res;
-
-	g_debug ("creating new namespace");
-
-	res = unshare (CLONE_NEWNS);
-	if (res != 0) {
-		g_print ("Failed to create new namespace: %s\n", strerror(errno));
-		return 1;
-	}
-
-	g_debug ("mount (private)");
-	mount_count = 0;
-	res = mount (LI_SW_ROOT_PREFIX, LI_SW_ROOT_PREFIX,
-				 NULL, MS_PRIVATE, NULL);
-	if (res != 0 && errno == EINVAL) {
-		/* Maybe if failed because there is no mount
-		 * to be made private at that point, lets
-		 * add a bind mount there. */
-		g_debug (("mount (bind)\n"));
-		res = mount (LI_SW_ROOT_PREFIX, LI_SW_ROOT_PREFIX,
-					 NULL, MS_BIND, NULL);
-		/* And try again */
-		if (res == 0) {
-			mount_count++; /* Bind mount succeeded */
-			g_debug ("mount (private)");
-			res = mount (LI_SW_ROOT_PREFIX, LI_SW_ROOT_PREFIX,
-						 NULL, MS_PRIVATE, NULL);
-		}
-	}
-
-	if (res != 0) {
-		g_error ("Failed to make prefix namespace private");
-		goto error_out;
-	}
-
-	return 0;
-
-error_out:
-	while (mount_count-- > 0)
-		umount (LI_SW_ROOT_PREFIX);
-	return 1;
-}
-
-/**
- * mount_overlay:
- * @pkgid: A software identifier (name-version)
- */
-static int
-mount_overlay (const gchar *pkgid)
+mount_app_bundle (const gchar *pkgid)
 {
 	int res = 0;
 	gchar *main_data_path = NULL;
 	gchar *fname = NULL;
-	gchar *wdir = NULL;
-	GFile *file;
+	g_autoptr(GFile) file = NULL;
 	const gchar *runtime_uuid;
 	gchar *tmp;
-	GError *error = NULL;
 	GString *lowerdirs = NULL;
 	LiPkgInfo *pki = NULL;
+	g_autofree gchar *newroot = NULL;
+	g_autofree gchar *approot_dir = NULL;
+	GError *error = NULL;
+
+	newroot = li_run_env_setup ();
+	if (newroot == NULL) {
+		res = 2;
+		goto out;
+	}
+	approot_dir = g_build_filename (newroot, "app", NULL);
 
 	/* check if the software exists */
 	main_data_path = g_build_filename (LI_SOFTWARE_ROOT, pkgid, "data", NULL);
@@ -181,13 +71,11 @@ mount_overlay (const gchar *pkgid)
 	if (!g_file_query_exists (file, NULL)) {
 		fprintf (stderr, "The software '%s' does not exist.\n", pkgid);
 		res = 1;
-		g_object_unref (file);
 		goto out;
 	}
 
 	pki = li_pkg_info_new ();
 	li_pkg_info_load_file (pki, file, &error);
-	g_object_unref (file);
 	if (error != NULL) {
 		fprintf (stderr, "Unable to read software metadata. %s\n", error->message);
 		goto out;
@@ -232,10 +120,10 @@ mount_overlay (const gchar *pkgid)
 	/* safeguard againt the case where only one path is set for lowerdir.
 	 * OFS doesn't like that, so we always set the root path as source too.
 	 * This also terminates the lowerdir parameter. */
-	g_string_append_printf (lowerdirs, "%s", LI_SW_ROOT_PREFIX);
+	g_string_append_printf (lowerdirs, "%s", approot_dir);
 
 	tmp = g_strdup_printf ("lowerdir=%s", lowerdirs->str);
-	res = mount ("overlay", LI_SW_ROOT_PREFIX,
+	res = mount ("overlay", approot_dir,
 				 "overlay", MS_MGC_VAL | MS_RDONLY | MS_NOSUID, tmp);
 	g_free (tmp);
 	if (res != 0) {
@@ -244,6 +132,11 @@ mount_overlay (const gchar *pkgid)
 		goto out;
 	}
 
+	if (!li_run_env_enter (newroot)) {
+		res = 3;
+		goto out;
+	}
+
 out:
 	if (lowerdirs != NULL)
 		g_string_free (lowerdirs, TRUE);
@@ -251,8 +144,6 @@ out:
 		g_free (main_data_path);
 	if (pki != NULL)
 		g_object_unref (pki);
-	if (wdir != NULL)
-		g_free (wdir);
 	if (error != NULL)
 		g_error_free (error);
 
@@ -297,7 +188,7 @@ main (gint argc, gchar *argv[])
 	GError *error = NULL;
 
 	/* ensue we have required capabilities, and drop all the ones we don't need */
-	if (!acquire_caps ()) {
+	if (!li_run_acquire_caps ()) {
 		g_printerr ("This program needs the suid bit to be set to function correctly.\n");
 		return 3;
 	}
@@ -324,16 +215,12 @@ main (gint argc, gchar *argv[])
 	swname = g_strdup (strv[0]);
 
 	/* create our environment */
-	ret = create_mount_namespace ();
-	if (ret > 0)
-		goto error;
-
-	ret = mount_overlay (swname);
+	ret = mount_app_bundle (swname);
 	if (ret > 0)
 		goto error;
 
 	/* Now we have everything we need CAP_SYS_ADMIN for, so drop that capability */
-	if (!drop_caps ()) {
+	if (!li_run_drop_caps ()) {
 		g_printerr ("Unable to drop capabilities.\n");
 		ret = 3;
 		goto error;
